% function to visualize the database
database = 'datasetSUN';
%database = 'dataset1';
setting = setEnvironment(database);
loadData;
load(fullfile(setting.path.database,'objectinfo.mat')); % load visualization information

%feature = 'gist';
feature = 'obj';
disp = 'seg';
%disp     = 'color';

% set the number of samples to visualize
switch database
    case 'dataset1'
        N = 800;
    case 'datasetSUN'
        N = 1200;
    otherwise
        error('Not recognized database');
end

% randomly select N samples
if exist('idx','var')~=1 || length(idx0)~=N
    idx = randperm(length(D));
    idx = idx(1:N);
    idx0 = idx;
else
    N = length(idx);
    idx = idx0;
end

fprintf('Compute distance...');
switch feature
    case 'gist'
        myGist = gist(idx,:);
        % compute the L2 distance
        myDist = pdist(myGist);
    case 'obj'
        myObj = sptHistObjects(idx,:);
        myDist = pdist(myObj,@distHistogram);
end
fprintf('done!\n');

% perform MDS 
fprintf('MDS...');
opt = statset('Display','iter','MaxIter',800);
Y = mdscale(myDist,2,'Start','random','Replicates',1,'Options',opt);
%Y = mdscale(myDist,2,'Criterion','metricstress','Replicates',5,'Options',opt);
Y0 = Y;
fprintf('done!\n');

% compute distance
Dist = squareform(myDist);
Y = Y0; idx = idx0;
% eliminate outlier
index1 = eliminateOutlier(Y(:,1));
index2 = eliminateOutlier(Y(:,2));
index = union(index1,index2);
if ~isempty(index)
    N = length(idx);
    index = setdiff(1:N,index);
    N = length(index);
    Dist = Dist(index,index);
    Y = Y(index,:);
    inlierIdx = idx(index);
end

[nKnn,Knn] = graphFromDist(Dist,10,mean(myDist));

% graph analysis
% knn = graphAnalysis(Knn);
% nEdges = ones(1,N);
% for i = 1:N
%     nEdges(i) = length(knn{i});
% end

rate = 3200/(max(Y(:,1)) - min(Y(:,1)));
Y(:,1) = Y(:,1)*rate;
Y(:,2) = Y(:,2)*rate*0.8;

Y = round(Y);

x0 = min(Y(:,1))-10;
y0 = min(Y(:,2))-10;
x1 = max(Y(:,1))+84;
y1 = max(Y(:,2))+84;


Im = ones(y1-y0+1,x1-x0+1,1);
width = x1-x0+1;
height = y1-y0+1;


% draw the edges of the graph
for i = 1:N
    for j = 1:nKnn(i)
        IDX = drawline([Y(i,2)-y0+32,Y(i,1)-x0+32],[Y(Knn{i}(j),2)-y0+32,Y(Knn{i}(j),1)-x0+32],[height,width]);
        Im(IDX) = 0.5; 
        %Im = func_Drawline(Im,Y(i,1)-x0,Y(i,2)-y0,Y(Knn{i}(j),1)-x0,Y(Knn{i}(j),2)-y0,0.5);
        %line([Y(i,1)-x0,Y(Knn{i}(j),1)-x0],[Y(i,2)-y0,Y(Knn{i}(j),2)-y0],'LineWidth',1,'Color',[0.5,0.5,0.5]);
    end
end


Im = imerode(Im,strel('disk',1));
Im = imfilter(Im,fspecial('gaussian',7,1.5),'same','replicate');

Edge = Im;


% add dots
[template,~,alpha] = imread('dotTemplate.png');
template = im2double(template);
alpha = im2double(alpha);
winsize = (size(template,1)-1)/2;
winlen = winsize*2+1;
for i = 1:N
    x1 = Y(i,1)-x0+32-winsize;
    y1 = Y(i,2)-y0+32-winsize;
    x2 = x1+winlen-1;
    y2 = y1+winlen-1;
    Im(y1:y2,x1:x2) = Im(y1:y2,x1:x2).*(1-alpha)+template.*alpha;
end
figure;imshow(Im);
imwrite(Im,[database '_Edge.jpg'],'quality',99);


dispList = {'color','seg','dot'};

for k = 1:3
    disp = dispList{k};
    
    Im = repmat(Edge,[1 1 3]);


    % plot images on the figure;
    template = im2double(imread('iconTemplate.png'));
    template = template.^1.5;

    for i = 1:N
        % load image
        switch disp
            case 'color'
                 im = LMimread(D,inlierIdx(i),setting.path.images);
                 im = im2double(imresize(im,0.25,'bicubic'));
            case 'seg'
                [foo,seg]=LM2segments(D(inlierIdx(i)),[],setting.path.images,setting.path.LMsegments);
                im = seg2color(seg,objectinfo.objcolormap);
                im = im2double(imresize(im,0.25,'bicubic'));
        end
         %image(Y(i,1),Y(i,2),im(end:-1:1,:,:));
         if size(im,3)==1
             im = repmat(im,[1 1 3]);
         end
         Im = addImToCanvas(im,[Y(i,1)-x0,Y(i,2)-y0],Im,template);
    end
    % Im = imresize(Im,0.7,'bicubic');
    figure;imshow(Im);
    imwrite(Im,[database '_canvas_' feature '_' disp '.jpg'],'quality',99);
    %saveas(gcf,[database '_canvas_' feature '_' disp '.eps'],'psc2');
end

return;

% code to recover idx
M = length(D);
for i  = 1:size(Y,1);
    %dist = sum(abs(repmat(myGist(i,:),[M,1]) - gist),2);
    dist = sum(abs(repmat(myObj(i,:),[M,1]) - sptHistObjects),2);
    [foo,idx(i)] = min(dist);
end

% eliminate the extreme ones
[foo,IDX]=max(Y(:,1));
Y = Y([1:IDX-1 IDX+1:end],:);
idx = idx([1:IDX-1 IDX+1:end]);
[foo,IDX]=min(Y(:,1));
Y = Y([1:IDX-1 IDX+1:end],:);
idx = idx([1:IDX-1 IDX+1:end]);
