% This script shows how to use the databases
%
% The main functions to use are:
%    LMdatabase to create the labelme structures and the indices
%    LMgist to compute the gist descriptor
%    LMdenseSift to compute the SIFT descriptors

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

setEnvironment;

% Read database structure
D = LMdatabase(setting.path.annotation);

% Read gist features
gist = LMgist(D, setting.path.images, setting.GISTparam, setting.path.gist);

% Read test and training indices and split datasets
load (fullfile(setting.path.database, 'splitTrainingTest'))
Dtest = D(testndx);
gistTest = gist(testndx, :);

Dtraining = D(trainingndx);
gistTraining = gist(trainingndx, :);

% Compute K-nearest neighbors
m = fix(length(Dtest)*rand)+1; % select randomly one image and get neighbors using GIST
knn = LMgistquery(gistTest(m,:), gistTraining);
knn = knn(1:30);

% Visualize neighbors with annotations
figure; 
LMplot(Dtest, m, setting.path.images); title('QUERY IMAGE');
LMdbshowscenes(Dtraining(knn), setting.path.images);


% Load SIFTs
% Visualize SIFTs for GIST neighbors
figure
for i = 1:30
    %sift = LMdenseSift(Dtraining(knn(i)), setting.path.images, setting.SIFTparam, HOMESIFT);
    im = LMimread(Dtraining,knn(i),setting.path.images);
    sift = mexDenseSIFT(im,setting.SIFTparam.cell_size,setting.SIFTparam.grid_spacing,1);
    subplot(5,6,i)
    showColorSIFT(sift/255);
end

% Visualize Segments for the neighbors
figure
for i = 1:30
    [tmp, seg, names] = LM2segments(Dtraining(knn(i)), [], setting.path.images, setting.path.LMsegments);
    subplot(5,6,i)
    %image(2*mod(seg,128)); 
    imshow(seg,[]);
    axis('equal'); 
    axis('tight');
    axis('off')
end

