#include "PairwiseBP.h"
#include "stdio.h"

PairwiseBP::PairwiseBP(void)
{
	IsDisplay=false;
	IsTRW=false;
	//IsTRW=true;
	CTRW=0.5;
	pDataTerm=NULL;
	pMessage=NULL;
	pBelief=NULL;
	ptrDataTerm=NULL;
	ptrMessage=NULL;
	ptrBelief=NULL;
	Width=Height=Area=nDims=0;
}

PairwiseBP::~PairwiseBP(void)
{
	ReleaseBuffer();
}

//----------------------------------------------------------------------------------------------------
// release buffers
//----------------------------------------------------------------------------------------------------
void PairwiseBP::ReleaseBuffer()
{
	// remove the buffer of the vectors
	_Release1DBuffer(ptrDataTerm);
	_Release1DBuffer(ptrMessage);
	_Release1DBuffer(ptrBelief);

	// null the buffer of every vector
	if(pDataTerm!=NULL)
		for(int i=0;i<Area;i++)
			pDataTerm[i].data()=NULL;
	if(pMessage!=NULL)
		for(int i=0;i<Area*4;i++)
			pMessage[i].data()=NULL;
	if(pBelief!=NULL)
		for(int i=0;i<Area;i++)
			pBelief[i].data()=NULL;

	// remove the buffer of the vectors
	_Release1DBuffer(pDataTerm);
	_Release1DBuffer(pMessage);
	_Release1DBuffer(pBelief);
}

//----------------------------------------------------------------------------------------------------
// function to allocate buffers for the vectors; notice that the a whole chunck of buffer is 
// allocated for each vector array
//----------------------------------------------------------------------------------------------------
void PairwiseBP::AllocateBuffer(int width, int height, int ndims)
{
	ReleaseBuffer();

	Width=width;
	Height=height;
	Area=Width*Height;
	nDims=ndims;
	
	// allocate the vectors
	pDataTerm=new Vector<T_message>[Area];
	pMessage=new Vector<T_message>[Area*4];
	pBelief=new Vector<T_message>[Area];
	
	// allocate the buffer for the vectors
	ptrDataTerm=new T_message[Area*nDims];
	ptrMessage=new T_message[Area*nDims*4];
	ptrBelief=new T_message[Area*nDims];
	memset(ptrDataTerm,0,sizeof(T_message)*Area*nDims);
	memset(ptrMessage,0,sizeof(T_message)*Area*nDims*4);
	memset(ptrBelief,0,sizeof(T_message)*Area*nDims);

	// assign the pointer of each vector
	for(int i=0;i<Area;i++)
	{
		pDataTerm[i].setDim(nDims);
		pDataTerm[i].data()=ptrDataTerm+i*nDims;
	}
	for(int i=0;i<Area*4;i++)
	{
		pMessage[i].setDim(nDims);
		pMessage[i].data()=ptrMessage+i*nDims;
	}
	for(int i=0;i<Area;i++)
	{
		pBelief[i].setDim(nDims);
		pBelief[i].data()=ptrBelief+i*nDims;
	}
	Im_x.allocate(Width,Height);
	message.allocate(nDims);
}

void PairwiseBP::LoadSmoothness(double d)
{
	if(Area==0)
	{
		cout<<"Call function LoadDataTerm() first!"<<endl;
		return;
	}
	Im_d.allocate(Width,Height,2);
	Im_d.setValue(d);
}

//----------------------------------------------------------------------------------------------------
// function for message passing
//----------------------------------------------------------------------------------------------------
double PairwiseBP::MessagePassing(int nIterations,int nHierarchy,double* pEnergyList)
{
    if(nHierarchy>0)
    {
        PairwiseBP bp;
        generateCoarserLevel(bp);
        bp.MessagePassing(5,nHierarchy-1);
        bp.propagateFinerLevel(*this);
    }
    
	double energy;

	ComputeBelief();
	energy=GetEnergy();
    if(IsDisplay)
        cout<<"Initial energy: "<<energy<<endl;

	for(int count=0;count<nIterations;count++)
	{
		BP_S(count);

		ComputeBelief();

		energy=GetEnergy();
		
		if(IsDisplay)
			cout<<"No. "<<count<<" energy: "<<energy<<endl;

		if(pEnergyList!=NULL)
			pEnergyList[count]=energy;
	}
	return energy;
}

//----------------------------------------------------------------------------------------------------
// function for sequential message update
//----------------------------------------------------------------------------------------------------
void PairwiseBP::BP_S(int count)
{
	if(count%2==0)
		for(int i=0;i<Height;i++)
			for(int j=0;j<Width;j++)
			{
				UpdateMessage(j,i,0);
				UpdateMessage(j,i,2);
			}
	else
		for(int i=Height-1;i>=0;i--)
			for(int j=Width-1;j>=0;j--)
			{
				UpdateMessage(j,i,1);
				UpdateMessage(j,i,3);
			}
}

//----------------------------------------------------------------------------------------------------
//  update the message from (x,y) to its neighbor with direction
//    the encoding of the direction
//               2  |
//                   v
//    0 ------> <------- 1
//                   ^
//                3 |
//----------------------------------------------------------------------------------------------------
void PairwiseBP::UpdateMessage(int x, int y, int direction)
{
	// eliminate impossible messages
	if (direction==0 && x==Width-1)
		return;
	if (direction==1 && x==0)
		return;
	if (direction==2 && y==Height-1)
		return;
	if (direction==3 && y==0)
		return;

	int offset=y*Width+x;

	double d;
	int x1=x,y1=y; // get destination
	switch(direction){
		case 0:
			x1++;
			d=Im_d.data()[offset*2];
			break;
		case 1:
			x1--;
			d=Im_d.data()[(offset-1)*2];
			break;
		case 2:
			y1++;
			d=Im_d.data()[offset*2+1];
			break;
		case 3:
			y1--;
			d=Im_d.data()[(offset-Width)*2+1];
			break;
	}

	// initialize the message from the data term
	message=pDataTerm[offset];

	if(IsTRW)
		message*=CTRW;

	// add spatial messages
	if(!IsTRW)
	{
		if(x>0 && direction!=1) // add left to right
			message+=pMessage[offset*4];
		if(x<Width-1 && direction!=0) // add right to left
			message+=pMessage[offset*4+1];
		if(y>0 && direction!=3) // add top down
			message+=pMessage[offset*4+2];
		if(y<Height-1 && direction!=2) // add bottom up
			message+=pMessage[offset*4+3];
	}
	else
	{
		if(x>0)
			if(direction==1)
				message.add(pMessage[offset*4],CTRW-1);
			else
				message.add(pMessage[offset*4],CTRW);
		if(x<Width-1) // add right to left
			if(direction==0)
				message.add(pMessage[offset*4+1],CTRW-1);
			else
				message.add(pMessage[offset*4+1],CTRW);
		if(y>0) // add top down
			if(direction==3)
				message.add(pMessage[offset*4+2],CTRW-1);
			else
				message.add(pMessage[offset*4+2],CTRW);
		if(y<Height-1) // add bottom up
			if(direction==2)
				message.add(pMessage[offset*4+3],CTRW-1);
			else
				message.add(pMessage[offset*4+3],CTRW);
	}
	// use distance transform to impose smoothness compability
	message._min(message.min()+d);

	// normalize the message
	message-=message.min();

	// assign the message to the right destination
	pMessage[(y1*Width+x1)*4+direction]=message;
}

//----------------------------------------------------------------------------------------------------
//  function to compute belief and do MAP inference
//----------------------------------------------------------------------------------------------------
void PairwiseBP::ComputeBelief()
{
	for(int i=0;i<Height;i++)
		for(int j=0;j<Width;j++)
		{
			int offset=i*Width+j;
			message=pDataTerm[offset];
			if(j>0)
				message+=pMessage[offset*4];
			if(j<Width-1)
				message+=pMessage[offset*4+1];
			if(i>0)
				message+=pMessage[offset*4+2];
			if(i<Height-1)
				message+=pMessage[offset*4+3];
			pBelief[offset]=message;
			Im_x.data()[offset]=message.minIdx();
		}
	FindOptimalSolutionSequential();
}

void PairwiseBP::FindOptimalSolutionSequential()
{
	double d;
	for(int i=0;i<Height;i++)
		for(int j=0;j<Width;j++)
		{
			int offset=i*Width+j;
			message=pDataTerm[offset];
			if(j>0)
			{
				d=Im_d.data()[(offset-1)*2];
				for(int l=0;l<nDims;l++)
					message[l]+=(l!=Im_x.data()[offset-1])*d;
			}
			if(i>0)
			{
				d=Im_d.data()[(offset-Width)*2+1];
				for(int l=0;l<nDims;l++)
					message[l]+=(l!=Im_x.data()[offset-Width])*d;
			}
			if(j<Width-1)
				message+=pMessage[offset*4+1];
			if(i<Height-1)
				message+=pMessage[offset*4+3];
			Im_x.data()[offset]=message.minIdx();
		}
}

//----------------------------------------------------------------------------------------------------
// function to compute energy
//----------------------------------------------------------------------------------------------------
double PairwiseBP::GetEnergy()
{
	double total=0;
	for(int i=0;i<Height;i++)
		for(int j=0;j<Width;j++)
		{
			int offset=i*Width+j;
			total+=pDataTerm[offset].data()[Im_x.data()[offset]];
			if(j<Width-1)
				if(Im_x.data()[offset]!=Im_x.data()[offset+1])
					total+=Im_d.data()[offset*2];
			if(i<Height-1)
				if(Im_x.data()[offset]!=Im_x.data()[offset+Width])
					total+=Im_d.data()[offset*2+1];
		}
	return total;
}


//------------------------------------------------------------------------------------------------
// multi-grid belie propagation
//------------------------------------------------------------------------------------------------
void PairwiseBP::generateCoarserLevel(PairwiseBP &bp)
{
	//------------------------------------------------------------------------------------------------
	// set the dimensions and parameters
	//------------------------------------------------------------------------------------------------
	bp.Width=Width/2;
	if(Width%2==1)
		bp.Width++;

	bp.Height=Height/2;
	if(Height%2==1)
		bp.Height++;
    
   bp.AllocateBuffer(bp.Width,bp.Height,nDims);

    bp.IsDisplay=IsDisplay;
    bp.IsTRW=IsTRW;
    bp.CTRW=CTRW;
    
    bp.Im_d.allocate(bp.Width,bp.Height,2);
    ReduceImage(bp.Im_d.data(),Width,Height,2,Im_d.data());
    
    for(int i=0;i<bp.Height;i++)
        for(int j=0;j<bp.Width;j++)
        {
            int offset=i*bp.Width+j;
            for(int ii=0;ii<2;ii++)
                for(int jj=0;jj<2;jj++)
                {
                    int y=i*2+ii;
                    int x=j*2+jj;
                    if(y<Height && x<Width)
                        bp.pDataTerm[offset]+=pDataTerm[y*Width+x];
                }
        }
}

//------------------------------------------------------------------------------------------------
// propagate from a coarse level  (this) to a finer one (bp)
//------------------------------------------------------------------------------------------------
void PairwiseBP::propagateFinerLevel(PairwiseBP& bp)
{
    for(int i=0;i<bp.Height;i++)
        for(int j=0;j<bp.Width;j++)
        {
            int y=i/2;
            int x=j/2;
            int offset=(y*Width+x)*4;
            int offset1=(i*bp.Width+j)*4;
            for(int k=0;k<4;k++)
                bp.pMessage[offset1+k]=pMessage[offset+k];
        }
}

//------------------------------------------------------------------------------------------------
// function to reduce the image 
//------------------------------------------------------------------------------------------------
template<class T>
void PairwiseBP::ReduceImage(T* pDstData,int width,int height,int nchannels,const T *pSrcData)
{
	int DstWidth=width/2;
	if(width%2==1)
		DstWidth++;
	int DstHeight=height/2;
	if(height%2==1)
		DstHeight++;
	memset(pDstData,0,sizeof(T)*DstWidth*DstHeight*nchannels);
	int sum=0;
	for(int i=0;i<DstHeight;i++)
		for(int j=0;j<DstWidth;j++)
		{
			int offset=(i*DstWidth+j)*nchannels;
			sum=0;
			for(int ii=0;ii<2;ii++)
				for(int jj=0;jj<2;jj++)
				{
					int x=j*2+jj;
					int y=i*2+ii;
					if(y<height && x<width)
					{
                        for(int l=0;l<nchannels;l++)
                            pDstData[offset+l]+=pSrcData[(y*width+x)*nchannels+l];
						sum++;
					}
				}
            for(int l=0;l<nchannels;l++)
                pDstData[offset+l]/=sum;
		}
}