#pragma once

#include "Vector.h"
#include "Image.h"
#include "memory.h"

typedef double T_message;  // T_message is the data type of the messages and beliefs


//-------------------------------------------------------------------
// the class for pairwise belief propagation
//-------------------------------------------------------------------

class PairwiseBP
{
private:
	bool IsDisplay;
	bool IsTRW;
	double CTRW;

	size_t Width,Height,nDims,Area; // defining the dimension of the grid
	Vector<T_message>* pDataTerm;
	Vector<T_message>* pMessage;
	Vector<T_message>* pBelief;
	Vector<T_message> message;
	T_message* ptrDataTerm;
	T_message* ptrMessage;
	T_message* ptrBelief;
	Image<T_message> Im_d;
	Image<int> Im_x;
public:
	PairwiseBP(void);
	~PairwiseBP(void);

	void AllocateBuffer(int width,int height,int ndims);
	void ReleaseBuffer();

	template <class T>
	void LoadDataTerm(int numObjects,const Image<T>* pImDataTerm);

	template <class T>
	void LoadDataTerm(const Image<T>& refImDataTerm);

	void LoadSmoothness(double d);

	template <class T>
	void LoadSmoothness(const Image<T>& im_d){
		if(!im_d.matchDimension(Width,Height,2))
			cout<<"The smoothness image must have the dimension [Width x Height x 2]!"<<endl;
		Im_d=im_d;
		}
	double MessagePassing(int nIterations,int nHierarchy,double* pEnergyList=NULL);

	void UpdateMessage(int x,int y,int direction);

	void ComputeBelief();
	
	void FindOptimalSolutionSequential();

	double GetEnergy();

	void BP_S(int count);

//----------------------------------------------------------------------------------------------------
// function to output the optimal solution
//----------------------------------------------------------------------------------------------------
#ifndef _MATLAB	
	bool OutputResult(char* filename)
	{
		 Im_x.writeImage(filename);
	}
#endif

	const Image<int>& output(){return Im_x;};
    
//------------------------------------------------------------------------
	// multi-grid belief propagation
	void generateCoarserLevel(PairwiseBP& bp);
	void propagateFinerLevel(PairwiseBP& bp);	
		
	template<class T>
	void ReduceImage(T* pOutputData,int width,int height,int nchannels,const T* pInputData);    
};

template <class T>
void PairwiseBP::LoadDataTerm(int numObjects, const Image<T> *pImDataTerm)
{
	AllocateBuffer(pImDataTerm[0].width(),pImDataTerm[0].height(),numObjects);	

	for(int i=0;i<nDims;i++)
		for(int j=0;j<Area;j++)
			pDataTerm[j].data()[i]=pImDataTerm[i].data()[j];
	for(int j=0;j<Area;j++)
		pDataTerm[j]-=pDataTerm[j].min();
}

template <class T>
void PairwiseBP::LoadDataTerm(const Image<T> &refImDataTerm)
{
	AllocateBuffer(refImDataTerm.width(),refImDataTerm.height(),refImDataTerm.nchannels());
	for(size_t j=0;j<Area;j++)
	{
		if(typeid(T)==typeid(T_message))
			memcpy(pDataTerm[j].data(),refImDataTerm.data()+j*nDims,sizeof(T_message)*nDims);
		else
			for(int i=0;i<nDims;i++)
				pDataTerm[j].data()[i]=refImDataTerm.data()[j*nDims+i];
	}
}