#pragma once

#include "memory.h"
#include <iostream>
#include "stdio.h"
#include "project.h"

using namespace std;

template <class T>
class Vector
{
private:
	int nDim;
	T* pData;
public:
	Vector(void);
	Vector(int ndim,T *data=NULL);
	Vector(const Vector& vect);
	~Vector(void);
	void releaseData();
	void allocate(int ndim);
	void allocate(const Vector<T>& vect){allocate(vect.nDim);};
	void copyData(const Vector<T>& vect);
	void dimcheck(const Vector<T>& vect) const;
	void reset();
	double norm2() const;
	void printVector();

	// access the members
	const T* data() const{return (const T*)pData;};
	T*& data() {return pData;};
	int dim() const {return nDim;};
	void setDim(int ndim) {nDim=ndim;};
	inline bool matchDimension(int _ndim) const {if(nDim==_ndim) return true;else return false;};
	inline bool matchDimension(const Vector<T> & vect) const {return matchDimension(vect.nDim);};

	// operators
	inline T operator[](int index) const {return pData[index];};
	inline T& operator[](int index){return *(pData+index);};
	Vector<T>& operator=(const Vector<T>& vect);
	
	Vector<T>& operator+=(const Vector<T>& vect);
	Vector<T>& operator*=(const Vector<T>& vect);
	Vector<T>& operator-=(const Vector<T>& vect);
	Vector<T>& operator/=(const Vector<T>& vect);

	Vector<T>& operator+=(double val);
	Vector<T>& operator*=(double val);
	Vector<T>& operator-=(double val);
	Vector<T>& operator/=(double val);

	T min() const{
		if(pData==NULL)
			return 0;
		T Min=pData[0];
		for(int i=1;i<nDim;i++)
			if(pData[i]<Min)
				Min=pData[i];
		return Min;
	}

	int minIdx() const{
		if(pData==NULL)
			return 0;
		T Min=pData[0];
		int idx=0;
		for(int i=1;i<nDim;i++)
			if(pData[i]<Min)
			{
				Min=pData[i];
				idx=i;
			}
		return idx;
	}


	void _min(T threshold){
		for(int i=0;i<nDim;i++)
			pData[i]=__min(pData[i],threshold);
	}

	Vector<T> operator *(double val)
	{
		Vector<T> result(nDim);
		for(int i=0;i<nDim;i++)
			result.pData[i]=pData[i]*val;
		return result;
	}
	
	void add(const Vector<T>& vect,double val)
	{
		for(int i=0;i<nDim;i++)
			pData[i]+=vect.pData[i]*val;
	}
	friend const Vector<T> operator+(const Vector<T>& vect1,const Vector<T>& vect2);
	friend const Vector<T> operator*(const Vector<T>& vect1,const Vector<T>& vect2);
	friend const Vector<T> operator-(const Vector<T>& vect1,const Vector<T>& vect2);
	friend const Vector<T> operator/(const Vector<T>& vect1,const Vector<T>& vect2);

	friend const Vector<T> operator+(const Vector<T>& vect1,double val);
	friend Vector<T> operator*(const Vector<T>& vect1,double val);
	friend const Vector<T> operator-(const Vector<T>& vect1,double val);
	friend const Vector<T> operator/(const Vector<T>& vect1,double val);

	friend double innerproduct(const Vector<T>& vect1,const Vector<T>& vect2);

	//bool writeVector(QFile& file) const;
	//bool readVector(QFile& file);
};


template <class T>
Vector<T>::Vector(void)
{
	nDim=0;
	pData=NULL;
}

template <class T>
Vector<T>::Vector(int ndim, T *data)
{
	nDim=ndim;
	pData=new double[nDim];
	if(data!=NULL)
		memcpy(pData,data,sizeof(T)*nDim);
	else
		memset(pData,0,sizeof(T)*nDim);
}

template <class T>
Vector<T>::Vector(const Vector<T>& vect)
{
	nDim=0;
	pData=NULL;
	copyData(vect);
}

template <class T>
Vector<T>::~Vector(void)
{
	releaseData();
}

template <class T>
void Vector<T>::releaseData()
{
	if(pData!=NULL)
		delete pData;
	pData=NULL;
	nDim=0;
}

template <class T>
void Vector<T>::allocate(int ndim)
{
	releaseData();
	nDim=ndim;
	if(nDim>0)
		pData=new T[nDim];
}

template <class T>
void Vector<T>::copyData(const Vector<T> &vect)
{
	if(nDim!=vect.nDim)
	{
		releaseData();
		nDim=vect.nDim;
		pData=new T[nDim];
	}
	memcpy(pData,vect.pData,sizeof(T)*nDim);
}

template <class T>
void Vector<T>::dimcheck(const Vector<T> &vect) const
{
	if(nDim!=vect.nDim)
		cout<<"The dimensions of the vectors don't match!"<<endl;
}

template <class T>
void Vector<T>::reset()
{
	if(pData!=NULL)
		memset(pData,0,sizeof(T)*nDim);
}

template <class T>
double Vector<T>::norm2() const
{
	double temp=0;
	for(int i=0;i<nDim;i++)
		temp+=pData[i]*pData[i];
	return temp;
}

template <class T>
void Vector<T>::printVector()
{
	for(int i=0;i<nDim;i++)
		cout<<pData[i]<<" ";
	cout<<endl;
}

//----------------------------------------------------------------------------------
// operators
//----------------------------------------------------------------------------------
template <class T>
Vector<T>& Vector<T>::operator =(const Vector<T> &vect)
{
	copyData(vect);
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator +=(const Vector<T> &vect)
{
	dimcheck(vect);
	for(int i=0;i<nDim;i++)
		pData[i]+=vect.data()[i];
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator *=(const Vector<T> &vect)
{
	dimcheck(vect);
	for(int i=0;i<nDim;i++)
		pData[i]*=vect.data()[i];
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator -=(const Vector<T> &vect)
{
	dimcheck(vect);
	for(int i=0;i<nDim;i++)
		pData[i]-=vect.data()[i];
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator /=(const Vector<T> &vect)
{
	dimcheck(vect);
	for(int i=0;i<nDim;i++)
		pData[i]/=vect.data()[i];
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator +=(double val)
{
	for(int i=0;i<nDim;i++)
		pData[i]+=val;
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator *=(double val)
{
	for(int i=0;i<nDim;i++)
		pData[i]*=val;
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator -=(double val)
{
	for(int i=0;i<nDim;i++)
		pData[i]-=val;
	return *this;
}

template <class T>
Vector<T>& Vector<T>::operator /=(double val)
{
	for(int i=0;i<nDim;i++)
		pData[i]/=val;
	return *this;
}

