// mex function to integrate annotations with belief propagation
#include "mex.h"
#include "project.h"
#include "Image.h"
#include "PairwiseBP.h"

double* outputMatrix2(mxArray*& plhs,int n1,int n2)
{
	int dims[2];
	dims[0]=n1;
	dims[1]=n2;
	plhs=mxCreateNumericArray(2,dims,mxDOUBLE_CLASS,mxREAL);
	return (double *)mxGetData(plhs);
}

double* outputMatrix3(mxArray*& plhs,int n1,int n2,int n3)
{
	int dims[3];
	dims[0]=n1;
	dims[1]=n2;
	dims[2]=n3;
	plhs=mxCreateNumericArray(3,dims,mxDOUBLE_CLASS,mxREAL);
	return (double *)mxGetData(plhs);
}

double* outputMatrix4(mxArray*& plhs,int n1,int n2,int n3,int n4)
{
	int dims[4];
	dims[0]=n1;
	dims[1]=n2;
	dims[2]=n3;
	dims[3]=n4;
	plhs=mxCreateNumericArray(4,dims,mxDOUBLE_CLASS,mxREAL);
	return (double *)mxGetData(plhs);
}


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	DImage Im_DataTerm;
	Im_DataTerm.LoadMatlabImage(prhs[0]);
    
    // define parameters
    double d=2;
    int nIterations=60;
    int nHierarchy=2;
    
    // load the parameters for matching
    if(nrhs>=2)
    {
        int nDims=mxGetNumberOfDimensions(prhs[1]);
		if(nDims>2)
			mexErrMsgTxt("The third parameter must be a vector!");
		const int* dims=mxGetDimensions(prhs[1]);
		if(dims[0]!=1 && dims[1]!=1)
			mexErrMsgTxt("The third parameter must be a vector!");
		int nPara=dims[0]+dims[1]-1;
        double* para=(double *)mxGetData(prhs[1]);
        
        if(nPara>=1)
            d=para[0];
        if(nPara>=2)
            nIterations=para[1];
        if(nPara>=3)
            nHierarchy=para[2];
    }
    cout<<"d: "<<d<<"    nIterations: "<<nIterations<<"    nHierarchy: "<<nHierarchy<<endl;
    
    PairwiseBP bp;
    bp.LoadDataTerm(Im_DataTerm);
    
	// load the smoothness term
    if(nrhs>=3)
    {
		DImage Im_d;
        Im_d.LoadMatlabImage(prhs[2]);
		bp.LoadSmoothness(Im_d);
    }
	else
		bp.LoadSmoothness(d);

	// output parameters
	double* pEnergyList=NULL;
	if(nlhs>1)
		pEnergyList=outputMatrix2(plhs[1],1,nIterations);
        
	// the core function is here
	bp.MessagePassing(nIterations,nHierarchy,pEnergyList);

	// output to matlab
	bp.output().OutputToMatlab(plhs[0]);
}