% the procedure to learn priors for the objects
% written by Ce Liu

% set the path and parameters

loadData;

% first count the object names

for m=1:length(D)
    fprintf('Processing sample %d\n',m);
    % load the info for 
    [foo,seg,names]=LM2segments(D(m),[],setting.path.images,setting.path.LMsegments);
    im = LMimread(D,m,setting.path.images);
    [height,width,nchannels]=size(im);
    if nchannels == 1
        im = repmat(im,[1 1 3]);
        nchannels = 3;
    end
    if m==1
        nClasses=length(names);
        counts=zeros(nClasses+1,1);
        for j=1:nClasses+1
            object(j).pixels=[];
            object(j).npixels=0;
            object(j).imprior=ones(height,width)*0.2;
        end
    end
    for i=0:nClasses
        mask=(seg==i);
        counts(i+1)=counts(i+1)+sum(mask(:));
    end
    
    % compute the median of the color

    im=reshape(im,[height*width,nchannels]);

    labels = segcount(seg);
    
    for i = labels
        segidx=find(seg==i);
        if ~isempty(segidx)
            %object(idx(i)).pixels=[object(idx(i)).pixels;median(im(segidx,:),1)];
            object(i+1).pixels=[object(i+1).pixels;sum(im(segidx,:),1)];
            object(i+1).npixels=object(i+1).npixels+length(segidx);
            object(i+1).imprior=object(i+1).imprior+(seg==i);
        end
    end
end

% plot the object priors
counts=counts/sum(counts);
[foo,idx]=sort(counts,'descend');
Counts=counts(idx);

nClasses=nClasses+1;
if length(names)<length(counts)
    names=['unlabeled' names];
end

% calculate the average color of each class
for i=1:nClasses
    %object(i).color=median(object(i).pixels,1);
    object(i).color=sum(object(i).pixels,1)/(object(i).npixels*255);
end
Object=object(idx);

% increase saturation and lower the value of the color
objcolormap=zeros(nClasses,3);
saturationBoost = 10;
valueBoost = 2;
for i=1:nClasses
    if isempty(object(i).color)
        objcolormap(i,:) = [0.5, 0.5, 0.5];
    else
        objcolormap(i,:)=adjustSaturation(object(i).color,saturationBoost,valueBoost);
        if strcmpi(names{i},'sea')
            objcolormap(i,:)=adjustSaturation(object(i).color,saturationBoost,1.4);
        end
        if strcmpi(names{i},'mountain')
            objcolormap(i,:)=adjustSaturation(object(i).color,10,0.75);
        end
    end
end
objcolormap(1,:) = [0.3,0.3,0.3]; % setting the color for the "unlabeled" class

map = objcolormap(idx,:);
if strcmp(setting.database,'datasetSUN');
    map(15:end,:) = max(min(map(15:end,:)+randn(size(map(15:end,:)))*0.3,1),0);
    objcolormap(idx,:) = map;
end

Names = names(idx);

% plot the histogram (both counts and corresponding potentials)
dispObjCounts(setting.database,Counts,Names,map,100,'object_counts_prior');
dispObjCounts(setting.database,-log(Counts+eps),Names,map,100,'object_counts_potential');



% plot spatial prior and save it
figure;
imbg=ones(height,width,3);
if exist(fullfile(setting.path.eps,'spatialprior'),'dir')~=7
    mkdir(fullfile(setting.path.eps,'spatialprior'));
end
for i=1:nClasses
    if(size(Object(i).imprior,3)==1)
        imprior=Object(i).imprior/max(Object(i).imprior(:));
        Object(i).imprior=reshape(kron(imprior(:),map(i,:)),[height,width,3]);
    end
    alpha=reshape(kron(imprior(:),[1 1 1]),[height,width,3]);
    clf;
    imshow(imaddboundary(Object(i).imprior.*alpha+imbg.*(1-alpha)));drawnow;
    saveas(gcf,fullfile(setting.path.eps, 'spatialprior', [Names{i} '.eps']),'psc2');
end

% save the prior information 
for i=1:nClasses
    prior=imfilter(object(i).imprior,fspecial('gaussian',25,5),'same','replicate');
    writeImage(prior,fullfile(setting.path.prior,[names{i} '.dat']));
end
fid=fopen(fullfile(setting.path.prior,'names.txt'),'w');
for i=1:nClasses
    fprintf(fid,'%s\r\n',names{i});
end
fclose(fid);

objprior=zeros(256,256,nClasses);
for i=1:nClasses
    %objprior(:,:,i)=-log(cropImage(object(i).imprior,setting.SIFTparam.cell_size))+8;
    objprior(:,:,i) = -log(object(i).imprior+eps)+8;
end
save(fullfile(setting.path.database,'objectPrior.mat'),'objprior','names');

% save object color information
objectinfo.names = names;
objectinfo.objcolormap = objcolormap;
objectinfo.counts = counts;
save(fullfile(setting.path.database,'objectinfo.mat'),'objectinfo');

showObjColorMap;
