function [prior_weight,spatial_weight]=objectRecogCoordDescend(database,varargin)

% set the environmental parameters
setting = setEnvironment(database);

%--------------------------------------------------------------------------------------------------------
% set the optional parameters
%--------------------------------------------------------------------------------------------------------
paraInitial = {'method','gistL2',...
                         'epsilon',2,...
                            'nCandidates',7,...
                             'nNeighbors',40,...
                             'prior_weight',0.1,...
                             'spatial_weight',60,...
                             'isDisplay',false,...
                             'isPrint',true,...
                             'isSaveEps',false,...
                             'alpha',0.7};
para = parsingVarArgIn(varargin,paraInitial(1:2:end),paraInitial(2:2:end));

filename= fullfile(setting.path.recog.performanceNoRate,['method=' para.method ' alpha=' num2str(para.alpha) ...
                                       ' nCandidates=' num2str(para.nCandidates) ' K=' num2str(para.nNeighbors) '.mat']);
  
if exist(filename,'file')==2 % the file already exists
    foo=load(filename);
    if ~isfield(foo,'prior_weight_array')
        foo.prior_weight_array=0:0.01:0.2;
    end
    if ~isfield(foo,'spatial_weight_array')
        foo.spatial_weight_array=5:5:200;
    end
    para.prior_weight=foo.prior_weight_array(foo.prior_idx);
    para.spatial_weight=foo.spatial_weight_array(foo.spatial_idx);
    
    if foo.prior_idx>=1 && foo.prior_idx<=length(foo.prior_weight_array) &&...
       foo.spatial_idx>=1 && foo.spatial_idx<=length(foo.spatial_weight_array)
        fprintf('Already optimal for alpha=%f nCandidates=%d K=%d! rate=%f\n',para.alpha,para.nCandidates,para.nNeighbors,foo.recogArray(foo.prior_idx,foo.spatial_idx));
       prior_weight = para.prior_weight;
       spatial_weight = para.spatial_weight;
        return;
    end
    hoo=foo;
end

prior_weight_array=0:0.01:0.2;
spatial_weight_array=5:5:200;

nPrior=length(prior_weight_array);
nSpatial=length(spatial_weight_array);

% search for the right index
[foo,prior_idx]=min(abs(prior_weight_array-para.prior_weight));
para.prior_weight=prior_weight_array(prior_idx);

[foo,spatial_idx]=min(abs(spatial_weight_array-para.spatial_weight));
para.spatial_weight=spatial_weight_array(spatial_idx);

recogArray=zeros(nPrior,nSpatial);
isComputed=zeros(nPrior,nSpatial);

if exist('hoo','var')==1
    [m,n]=size(hoo.recogArray);
    recogArray(1:m,1:n)=hoo.recogArray;
    isComputed(1:m,1:n)=hoo.isComputed;
end

nNeighbors=2;

for k=1:10
    % save the current idx
    current_idx=[prior_idx,spatial_idx];
    
    % search along the prior dimension
    index=max(prior_idx-nNeighbors,1):min(prior_idx+nNeighbors,nPrior);
    for i=index
        if isComputed(i,spatial_idx)==0
            para.prior_weight=prior_weight_array(i);
            para.spatial_weight=spatial_weight_array(spatial_idx);
            fprintf('Prior_weight=%f Spatial_weight=%f\n',para.prior_weight,para.spatial_weight);
            paraCell = formVarargin(para);
            rate=objectRecognition(database,paraCell{:});
            fprintf('Rate=%f\n\n',rate);
            recogArray(i,spatial_idx)=rate;
            isComputed(i,spatial_idx)=1;
        end
    end
    [foo,idx]=max(recogArray(index,spatial_idx));
    prior_idx=index(idx);
    
    % search along the spatial dimension
    index=max(spatial_idx-nNeighbors,1):min(spatial_idx+nNeighbors,nSpatial);
    for i=index
        if isComputed(prior_idx,i)==0
            para.prior_weight=prior_weight_array(prior_idx);
            para.spatial_weight=spatial_weight_array(i);
            fprintf('Prior_weight=%f Spatial_weight=%f\n\n',para.prior_weight,para.spatial_weight);
            paraCell = formVarargin(para);
            rate=objectRecognition(database,paraCell{:});
            fprintf('Rate=%f\n\n',rate);
            recogArray(prior_idx,i)=rate;
            isComputed(prior_idx,i)=1;
        end
    end
    [foo,idx]=max(recogArray(prior_idx,index));
    spatial_idx=index(idx);
    
    close all;
    if current_idx==[prior_idx,spatial_idx]
        break;
    end
    
end

optimalRecogRate=recogArray(prior_idx,spatial_idx);
fprintf('\nOptimal rate: %f\n\n',optimalRecogRate);
para.spatial_weight=spatial_weight_array(spatial_idx);
para.prior_weight=prior_weight_array(prior_idx);

save(fullfile(setting.path.recog.performance,['method=' para.method ' alpha=' num2str(para.alpha) ' nCandidates=' num2str(para.nCandidates) ' K=' num2str(para.nNeighbors) ' prior=' num2str(para.prior_weight)...
      ' spatial=' num2str(para.spatial_weight) ' recograte=' num2str(optimalRecogRate) '.mat']),...
      'recogArray','prior_weight_array','spatial_weight_array','isComputed','prior_idx','spatial_idx');
save(filename,...
      'recogArray','prior_weight_array','spatial_weight_array','isComputed','prior_idx','spatial_idx');
  
prior_weight = para.prior_weight;
spatial_weight = para.spatial_weight;
