% function to plot recognition rate
function plotRecogRate(database,varargin)

% set the environmental parameters
% load the training and test data
setting = setEnvironment(database);
load(fullfile(setting.path.database,'objectinfo.mat')); % load visualization information
load(fullfile(setting.path.database,'objectPrior.mat')); %load matrix "objprior" and "names"

%--------------------------------------------------------------------------------------------------------
% set the optional parameters
%--------------------------------------------------------------------------------------------------------
optParaDefault = {'method','gistL2',...
                            'nCandidates',10,...
                             'nNeighbors',50,...
                             'prior_weight',0.1,...
                             'spatial_weight',60,...
                             'epsilon',10000,...
                             'recogRate',0.5,...
                             'alpha',0.7};
para = parsingVarArgIn(varargin,optParaDefault(1:2:end),optParaDefault(2:2:end));
filename =  ['ConfusionMatrix method=' para.method ' nCandidates=' num2str(para.nCandidates) ' K=' num2str(para.nNeighbors)...
                    ' epsilon=' num2str(para.epsilon)  ' spatial=' num2str(para.spatial_weight) ' prior=' num2str(para.prior_weight) '.mat'];
                
if exist(fullfile(setting.path.recog.home,filename),'file')~=2
    error(['The file ' filename ' does not exist!']);
end

load(fullfile(setting.path.recog.home,filename));


performance=diag(M);
%--------------------------------------------------------------------------------------------------------
% treat unlabeled
%--------------------------------------------------------------------------------------------------------
% sort the index
[foo,idx]=sort(objectinfo.counts,'descend');

for i=1:length(idx)
    if strcmp(names{idx(i)},'unlabeled')==1
        hh=i;
        break;
    end
end
idx=[idx(1:hh-1);idx(hh+1:end)];
performance=[performance(1:hh-1);performance(hh+1:end)];
Names=objectinfo.names(idx);
Colormap = objectinfo.objcolormap(idx,:);
if exist('recogRate','var')~=1
    recogRate=para.recogRate;
end

if strcmp(database,'dataset1') % if database = dataset1 then ignore the last three categories
     dispObjRecogRate(performance(1:end-3),recogRate,Names(1:end-3),Colormap(1:end-3,:));
else
    dispObjRecogRate(performance,recogRate,Names,Colormap);
end

saveas(gcf,fullfile(setting.path.recog.eps,['method=' para.method  ' alpha=' num2str(para.alpha) ' nCandidates=' num2str(para.nCandidates)...
                                ' K=' num2str(para.nNeighbors)  ' epsilon=' num2str(para.epsilon)...
                               ' spatial=' num2str(para.spatial_weight) ' prior=' num2str(para.prior_weight)...
                               ' RecogRate=' num2str(recogRate) '.eps']),'psc2');
