% function to plot the performance as a function of the training ratio
expRatio = -3:0.1:0;
trainingRatio = 10.^expRatio;

setting = setEnvironment('dataset1');
srcpath = fullfile(setting.path.database,'trainingRatio');

RecogRate = [];
Ratio = [];
for i = 1:length(expRatio)
    filename = fullfile(srcpath,['trainingRatio=' num2str(trainingRatio(i)) '.mat']);
    if exist(filename,'file') == 2
        load(filename);
        RecogRate(end+1,:) = recogRate;
        Ratio(end+1) = expRatio(i);
    else
        fprintf('%f %f\n',expRatio(i),trainingRatio(i));
    end
end

rate.mean = mean(RecogRate,2);
rate.std     = std(RecogRate,1,2);
figure;
errorbar(Ratio,rate.mean,rate.std,'Color',[0.2,0.2,0.4],'LineWidth',2,'MarkerSize',6,'MarkerFaceColor','r');hold on;
xlim([-3.1,1]);
ylim([0,1]);
line([0,0],[0,0.99],'LineStyle','--','Color',[0.5,0.5,0.5],'HandleVisibility','off');
line([-3,1],[1 1]*rate.mean(end),'LineStyle','--','Color',[0.6,0.5,0.5],'HandleVisibility','off');

% use the last 3 numbers to fit a line
n = 10;
Coeff = [Ratio(end-n+1:end)' ones(n,1)]\rate.mean(end-n+1:end);
line([0,1],[rate.mean(end) sum(Coeff)],'Color','g','LineWidth',2);

fprintf('Extrapolating to %f\n',sum(Coeff));

set(gca,'FontSize',24,'FontName','Times New Roman');
xlabel('log10(training ratio)');
legend('Increasing training ratio','Extrapolating','Location','SouthEast');

saveas(gcf,'trainingRatio.eps','psc2');