% procedure to set the environmental parameters for scene matching
% written by Ce Liu
% May 13, 2010
function setting = setEnvironment(database)

if exist('database','var')~=1
    database = 'datasetLMO';
end

switch database
    case 'datasetLMO'
    case 'datasetSUN'
    otherwise
        error(['Database "' database '" is unknown!']);
end

if ispc   % windows
    setting.path.home  = fileparts(pwd);
else if ismac % mac
        setting.path.home = fileparts(pwd);
    else % linux
        setting.path.home = fileparts(pwd);
    end
end

%setting.path.matlab = fullfile(setting.path.home,'matlab');
setting.path.matlab = pwd;

% take of the libraries
Paths = path;
if isempty(strfind(Paths,fullfile(setting.path.matlab,'toolboxLabelME')))
    addpath(fullfile(setting.path.matlab,'toolboxLabelME'));
end
if isempty(strfind(Paths,fullfile(setting.path.matlab,'mexIntegrateAnnotation')))
    addpath(fullfile(setting.path.matlab,'mexIntegrateAnnotation'));
end
if isempty(strfind(Paths,fullfile(setting.path.matlab,'warpImageInt')))
    addpath(fullfile(setting.path.matlab,'warpImageInt'));
end
if isempty(strfind(Paths,fullfile(setting.path.matlab,'SIFTflow')))
    addpath(fullfile(setting.path.matlab,'SIFTflow'));
end
if isempty(strfind(Paths,fullfile(setting.path.matlab,'mexDenseSIFT')))
    addpath(fullfile(setting.path.matlab,'mexDenseSIFT'));
end
if isempty(strfind(Paths,fullfile(setting.path.matlab,'library')))
    addpath(fullfile(setting.path.matlab,'library'));
end


% if ~isempty(strfind(Paths,fullfile(setting.path.matlab,'NewSIFTflow')))
%     rmpath(fullfile(HOMEFOLDER,'NewSIFTflow'));
% end
% if ~isempty(strfind(Paths,fullfile(setting.path.matlab,'SIFTflow')))
%     rmpath(fullfile(HOMEFOLDER,'SIFTflow'));
% end

% create folders
setting.database = database;
setting.path.database     = fullfile(setting.path.home,'datasets',database);
setting.path.images       = fullfile(setting.path.database,'Images');
setting.path.segmentation       = fullfile(setting.path.database,'Segmentation');
setting.path.annotation   = fullfile(setting.path.database,'Annotations');
setting.path.gist         = fullfile(setting.path.database,'Gist');
setting.path.LMsegments   = fullfile(setting.path.database,'LMsegments');
setting.path.likelihood   = fullfile(setting.path.database,'Likelihood');
setting.path.recog.home = fullfile(setting.path.database,'Recog');
setting.path.recog.performance = fullfile(setting.path.database,'performance');
setting.path.recog.performanceNoRate = fullfile(setting.path.database,'performanceNoRate');
setting.path.recog.jpg    = fullfile(setting.path.database,'RecogJPG');
setting.path.recog.eps    = fullfile(setting.path.database,'RecogEPS');
setting.path.matching.home = fullfile(setting.path.database,'Matching');
setting.path.matching.jpg  = fullfile(setting.path.database,'MatchingJPG');
setting.path.matching.eps  = fullfile(setting.path.database,'MatchingEPS');
setting.path.prior                 = fullfile(setting.path.database,'prior');
setting.path.eps                   = fullfile(setting.path.database,'eps');
% recursively create the directories of the fields in setting.path
structMkdir(setting.path);
    
%--------------------------------------------------------------------
% set the number of test images
%--------------------------------------------------------------------
if ~isempty(findstr('datasetLMO',setting.path.database))
    setting.para.nTest = 200;
end
if ~isempty(findstr('datasetSUN',setting.path.database))
    setting.para.nTest = 1000;
end

%-------------------------------------------------------------
% SOME PARAMETERS
%-------------------------------------------------------------

% gist
setting.GISTparam.imageSize                    = 256;
setting.GISTparam.orientationsPerScale = [8 8 8 8];
setting.GISTparam.numberBlocks            = 4;
setting.GISTparam.fc_prefilt                      = 4;

% hog
setting.VWparamhog.imageSize               = [256 256]; % it works also with non-square images
setting.VWparamhog.grid_spacing         = 1; % distance between grid centers
setting.VWparamhog.patch_size              = 16; % size of patch from which to compute SIFT descriptor (it has to be a factor of 4)
setting.VWparamhog.NumVisualWords = 200; % number of visual words
setting.VWparamhog.Mw                            = 2; % number of spatial scales for spatial pyramid histogram
setting.VWparamhog.descriptor               = 'hog'; 
setting.VWparamhog.w                                = floor(setting.VWparamhog.patch_size/2*2.5); % boundary for HOG


% dense sift
setting.SIFTparam.grid_spacing = 1; % distance between grid centers
setting.SIFTparam.cell_size    = 3; % size of cell from which to compute SIFT descriptor

% LCHparam.grid_spacing = 1; % distance between grid centers 
% LCHparam.patch_size = 16; % size of patch from which to compute SIFT descriptor (it has to be a factor of 4) 
% LCHparam.mum_color_bins = 16; 
% LCHparam.num_spt_bins = 1;

% the number of nearest neighbors of scene retrieval
setting.K                           = 20;

setting.IsMultiThread    = true;

% setting the KNN methods; 
setting.KNNmethod  = 'gistL2';
% setting.KNNmethod = 'gistCorr';
% setting.KNNmethod = 'hog';
% setting.KNNmethod = 'gt';
% setting.KNNmethod = 'all';

% parameters for SIFT flow
setting.SIFTflowpara.alpha          = 1*255;   
setting.SIFTflowpara.d              = setting.SIFTflowpara.alpha*20*255;
setting.SIFTflowpara.gamma          = 0.0001;
setting.SIFTflowpara.nlevels        = 4;
setting.SIFTflowpara.topwsize       = 20;
setting.SIFTflowpara.wsize          = 5;
setting.SIFTflowpara.nIterations    = 50; % increasing the number of iterations leads to higher accuracy yet more tme
setting.SIFTflowpara.nTopIterations = 100;

% SIFTflowpara.topwsize=20;
% SIFTflowpara.wsize=3;
% SIFTflowpara.nIterations=30;
% SIFTflowpara.nTopIterations=60;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load the data

% if exist('Dtraining','var')~=1
%     if exist('database.mat','file')~=2
%         % Read database structure
%         D = LMdatabase(HOMEANNOTATIONS);
% 
%         % Read gist features
%         gist = LMgist(D, HOMEIMAGES, GISTparam, HOMEGIST);
% 
%         % Read test and training indices and split datasets
%         load (fullfile(HOME, 'splitTrainingTest'))
%         save('database.mat','D','gist','testndx','trainingndx');
%     else
%         load('database.mat');
%     end
% 
%     Dtest = D(testndx);
%     gistTest = gist(testndx, :);
% 
%     Dtraining = D(trainingndx);
%     gistTraining = gist(trainingndx, :);
% end

%fprintf('Both training and test data are loaded!\n');
% data is loaded
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
