function [lch, LCHparam] = LMdenselocalcolorhist(D, HOMEIMAGES, LCHparam, HOMELCH)
%
% Antonio Torralba, 2008


if nargin==4
    precomputed = 1;
    % get list of folders and create non-existing ones
    % listoffolders = {D(:).annotation.folder};
else
    precomputed = 0;
    HOMELCH = '';
end

if nargin<3
    % Default parameters
    LCHparam.grid_spacing = 1; % distance between grid centers
    LCHparam.patch_size = 16; % size of patch from which to compute SIFT descriptor (it has to be a factor of 4)
    LCHparam.num_spt_bins = 2;
    LCHparam.mum_color_bins = 16;
end

Nfeatures = LCHparam.num_spt_bins^2 * LCHparam.mum_color_bins;

if isstruct(D)
    % [lch, LCHparam] = LMdenselocalcolorhist(D, HOMELCH, LCHparam);
    Nscenes = length(D);
    typeD = 1;
end
if iscell(D)
    % [lch, LCHparam] = LMdenselocalcolorhist(filename, HOMELCH, LCHparam);
    Nscenes = length(D);
    typeD = 2;
end
if isnumeric(D)
    % [lch, LCHparam] = LMdenselocalcolorhist(img, HOMELCH, LCHparam);
    Nscenes = size(D,4);
    typeD = 3;
end

if Nscenes >1
    fig = figure;
end

% Loop: Compute LCH features for all scenes
lch = zeros([Nscenes Nfeatures], 'single');
for n = 1:Nscenes
    g = [];
    todo = 1;

    % if LCH has already been computed, just read the file
    if precomputed==1
        filename = fullfile(HOMELCH, D(n).annotation.folder, [D(n).annotation.filename(1:end-4) '.mat']);
        if exist(filename, 'file')
            load(filename, 'lch', 'LCHparam');
            todo = 0;
        end
    end

    % otherwise compute LCH
    if todo==1
        disp([n Nscenes])

        % load image
        try
            switch typeD
                case 1
                    img = LMimread(D, n, HOMEIMAGES);
                case 2
                    img = imread(fullfile(HOMEIMAGES, D{n}));
                case 3
                    img = D(:,:,:,n);
            end
        catch
            disp(D(n).annotation.folder)
            disp(D(n).annotation.filename)
            rethrow(lasterror)
        end

        % get LCH descriptors
        lch = denselocalcolorhist(img, LCHparam);

        % save lch if a HOMELCH file is provided
        if precomputed
            mkdir(fullfile(HOMELCH, D(n).annotation.folder))
            save (filename, 'lch', 'LCHparam')
        end

        if Nscenes >1
            figure(fig);
            subplot(121)
            imshow(uint8(img))
            subplot(122)
            montage(reshape(lch, [size(lch,1) size(lch,2) 1 size(lch,3)]))
        end
    end

    drawnow
end




function [lch, grid_x, grid_y] = denselocalcolorhist(I, LCHparam)
%

% parameters
grid_spacing = LCHparam.grid_spacing;
patch_size = LCHparam.patch_size;
num_spt_bins = LCHparam.num_spt_bins;
mum_color_bins = LCHparam.mum_color_bins;

sigma = 256/sqrt(mum_color_bins);
w = patch_size/2; % boundary

% add boundary
I = padreflect(I, w-1);
if ~isfield(LCHparam, 'bincenters')
    n = ceil(sqrt(mum_color_bins));
    % sample
    mina = -32; maxa = 32;
    minb = -32; maxb = 32;
    sigma = (maxa-mina)/n/2;

    [a,b] = meshgrid(linspace(mina, maxa, n+1), linspace(minb, maxb, n+1));
    aa = (a(1:end-1,1:end-1) + a(1:end-1,2:end))/2;
    bb = (b(1:end-1,1:end-1) + b(2:end,1:end-1))/2;
    LCHparam.bincenters = [aa(:) bb(:)];
else
    n = ceil(sqrt(mum_color_bins));
    % sample
    mina = -32; maxa = 32;
    minb = -32; maxb = 32;
    sigma = (maxa-mina)/n/2;
end


nfeatures = num_spt_bins^2 * mum_color_bins;
I = double(I);
I = I /max(I(:));

[nrows ncols cc] = size(I);

lab = single(rgb2lab(I));


% compute bins and weight by luminance...
I_lch = zeros([nrows, ncols, mum_color_bins], 'single');

for k=1:mum_color_bins
    % compute each orientation channel
    tmp = exp(-((lab(:,:,2) - LCHparam.bincenters(k,1)).^2 + (lab(:,:,3) - LCHparam.bincenters(k,2)).^2) / (2*sigma^2));

    % weight by magnitude
    I_lch(:,:,k) = tmp .* lab(:,:,1);
end

% Convolution formulation:
weight_kernel = zeros(patch_size,patch_size);
r = patch_size/2;
cx = r - 0.5;
sample_res = patch_size/num_spt_bins;
weight_x = abs((1:patch_size) - cx)/sample_res;
weight_x = (1 - weight_x) .* (weight_x <= 1);

for k = 1:mum_color_bins
    I_lch(:,:,k) = conv2(weight_x, weight_x', I_lch(:,:,k), 'same');
end

% Sample SIFT bins at valid locations (without boundary artifacts)
% find coordinates of sample points (bin centers)
[sample_x, sample_y] = meshgrid(linspace(1,patch_size+1,num_spt_bins+1));
sample_x = sample_x(1:num_spt_bins,1:num_spt_bins); sample_x = sample_x(:)-patch_size/2;
sample_y = sample_y(1:num_spt_bins,1:num_spt_bins); sample_y = sample_y(:)-patch_size/2;

% grid
grid_x = patch_size/2:grid_spacing:ncols-patch_size/2+1;
grid_y = patch_size/2:grid_spacing:nrows-patch_size/2+1;

lch = zeros([length(grid_y) length(grid_x) nfeatures], 'single');
b = 0;
for n = 1:num_spt_bins*num_spt_bins
    lch(:,:,b+1:b+mum_color_bins) = I_lch(grid_y+sample_y(n), grid_x+sample_x(n), :);
    b = b+mum_color_bins;
end
clear I_lch


% Outputs:
[grid_x,grid_y] = meshgrid(grid_x, grid_y);
[nrows, ncols, cols] = size(lch);

% remove boundary
%lch = removereflect(lch, w);

% normalize LCH descriptors
ct = .1;
lch = lch + ct;
tmp = sqrt(sum(lch.^2, 3));
lch = lch ./ repmat(tmp, [1 1 size(lch,3)]);


function lab = rgb2lab(rgb)
%
% this function can be accelerated

C = makecform('srgb2lab');
lab = applycform(rgb,C);


function I = padreflect(I, w)

[ly lx c] = size(I);
I = [
    I(w+1:-1:2,w+1:-1:2,:), I(w+1:-1:2,:,:), ...
    I(w+1:-1:2,lx-1:-1:lx-w,:); ...
    I(:,w+1:-1:2,:),    I,   I(:,lx-1:-1:lx-w,:); ...
    I(ly-1:-1:ly-w,w+1:-1:2,:), ...
    I(ly-1:-1:ly-w,:,:), ...
    I(ly-1:-1:ly-w,lx-1:-1:lx-w,:) ];

function I = removereflect(I, w)

I = I(w+1:end-w, w+1:end-w,:);




